function [Qd,Qc,Qs,Cd,Cc,Ca,Cs] = nephron_model(Qd0,Cd0,Pd,D,N,par_nephron)
% Function INPUTS:
%   Qd0: input flow into the descending limb
%   Cd0: input concentration in to the descending limb
%   Pd: distal tubule (and descending limb) hydrostatic pressure
%   D: length of nephron
%   N: number of numerical grid points
%   par_nephron: list of parameter values
% Function OUTPUTS:
%   Qd: vector of descending limb flow
%   Qc: vector of collecting duct flow
%   Qs: vector of ascending v.r. flow
%   Cd, Cc, Ca, Cs: vectors of sodium concentrations

% Model parameters:
RT = 18.717; % mmHg mM^{-1}
Hd = par_nephron(1);
Hc = 0;
Kd = par_nephron(2);
Kc = par_nephron(3);
Pmax = par_nephron(4);
Kmp = par_nephron(5);
% np = par_nephron(6);
np = 1;
CaMAX = 500;
% Delta P's for Equations (11) and (13)
DeltaPd = Pd + 17 - 3; % Pd + Pi_s - P_s
DeltaPc = (Pd-1) + 17 - 3; % Pc + Pi_s - P_s
% Boundary condition (asdending v.r.)
Qs1 = 0;
Cs1 = Cd0;
% Discretization parameters
dx = D/N;
x = (0.5:1:N)'.*dx;
% Initialize integration operator
Int_pos = dx*tril(ones(N,N),0) + 0*(dx/2)*diag(ones(N,1),0);
% Initialize variables
Qd = Qd0*ones(N,1);
Qc = Qd0*ones(N,1)/2;
Cd = 1*Cd0*ones(N,1);
Cc = 1*Cd0*ones(N,1);
Cs = 1*Cd0*ones(N,1);
Ca = 1*Cd0*ones(N,1);
% More numerical procedure parameters
% (these are used in the iterative algorithm)
ep = 0.1;
E = 100;
Eprev = 100;
count = 0;
% This loop iterates until the error is below a tolerance or the number of
% iteration exceeds a maximum.
while (E > 1e-4) && (count < 10000)
  count = count + 1;
  % Equation (16):
  P = max(0,Pmax*Ca.^np./(Ca.^np + Kmp^np).*(CaMAX^5./(Ca.^5 + CaMAX^5)));
  % The following use Equation (A1) to iterate toward the solution.
  % The flows are assigned values
  %    Q_new = (1-ep)*Q_old + ep*Q_solution,
  % where Q_solution is obtained from (A1). The algorithm tends to be
  % stable when ep < 1.
  Qd = max(0,(1-ep)*Qd + ep*(Qd0 + Kd*Int_pos*(-DeltaPd + 2*RT*(Cd-Cs) )));
  Qa = -Qd(N);
  Qc0 = -Qa;
  Qc = max(0,(1-ep)*Qc + ep*(Qc0 + Kc*Int_pos*(-DeltaPc + 2*RT*(Cc-Cs) )));
  Qs = Qc(N) - (Qd+Qa+Qc) + Qs1;
  Qc = max(0,Qc);
  Qs = min(0,Qs);
   
  % Compute Na+ concentrations
  % (1.) ISF Cs equations for ascending VR exchange (Qs<0).
  %  Here Cs is determined from Equation (A5)
  Qs0 = Qs(1) - (Qd0 - (Qd(1)+Qa+Qc(1)));
  Cs(N) = (-Qs1*Cs1 + dx*(Hd*Cd(N)+Hc*Cc(N)+P(N))) / (-Qs(N-1) + dx*(Hd+Hc));
  for i = (N-1):-1:2
    Cs(i) = (-Qs(i)*Cs(i+1) + dx*(Hd*Cd(i)+Hc*Cc(i)+P(i))) / (-Qs(i-1) + dx*(Hd+Hc));
  end
  Cs(1) = (-Qs(1)*Cs(2) + dx*(Hd*Cd(1)+Hc*Cc(1)+P(1))) / (-Qs0 + dx*(Hd+Hc));
  Cs = max(Kmp/10,Cs);  
  % (2.) Descending limb. Here Cd is dtermined from Equation (A2)
  Cd(1) = (Qd0*Cd0 + dx*Hd*Cs(1)) / (Qd(1) + dx*Hd);
  for i = 2:N
    Cd(i) = (Qd(i-1)*Cd(i-1) + dx*Hd*Cs(i)) / (Qd(i) + dx*Hd);
  end
  Cd = min(2000,Cd);
  % (3.) Ascending limb. Here Ca is determined from Equation (A3)
  Ca1 = Cd(N);
  Ca(N) = Ca1 + dx*P(i)/Qa;
  for i = N-1:-1:1
    Ca(i) = Ca(i+1) + dx*P(i)/Qa;
  end
  Ca = min(max(Kmp/25,Ca),1000);
  % (4.) Collecting duct. Here Cd is determined from Equation (A4)
  Cc0 = Ca(1);
  Cc(1) = (Qc0*Cc0 + dx*Hc*Cs(1)) / (Qc(1) + dx*Hc);
  for i = 2:N
    Cc(i) = (Qc(i-1)*Cc(i-1) + dx*Hc*Cs(i)) / (Qc(i) + dx*Hc);
  end
  Cc = min(Cc,Cs);
  
  % Compute error of current solution: E1 and E2 are the norms of the
  % errors in the equations for Qd and Qc in Equation (A1)
  dQddx(2:N) = (Qd(2:N)-Qd(1:N-1))/dx;
  dQddx(1) = (Qd(1)-Qd0)/dx;
  E1 = norm(Kd*(-DeltaPd+2*RT*(Cd-Cs)) - dQddx');
  dQcdx(2:N) = (Qc(2:N)-Qc(1:N-1))/dx;
  dQcdx(1) = (Qc(1)-Qc0)/dx;
  E2 = norm(Kc*(-DeltaPc+2*RT*(Cc-Cs)) - dQcdx');
  E = E1 + E2;
  % The solution converges more quickly if ep is increased every so often:
  if mod(count,2000) == 0
    ep = 2*ep;
  end
  % However, ep must be reduced if the error is increasing.
  Ecurr = E;
  if Ecurr >= 1.015*Eprev
    ep = ep/1.1;
  end
  % If ep becomes very small, then the procedure can stall out. Therefore
  % when ep is reduced below some value, we re-initialize everything and
  % hope for the best:
  if ep < 1e-6
    ep = 1e-3;
    % Initialize variables
    Cd = 2*Cd0*ones(N,1);  
    Cc = 2*Cd0*ones(N,1);
    Cs = 2*Cd0*ones(N,1);
    Ca = 2*Cd0*ones(N,1);
    dQddx(2:N) = (Qd(2:N)-Qd(1:N-1))/dx;
    dQddx(1) = (Qd(1)-Qd0)/dx;
    E1 = norm(Kd*(-DeltaPd+RT*(Cd-Cs)) - dQddx');
    dQcdx(2:N) = (Qc(2:N)-Qc(1:N-1))/dx;
    dQcdx(1) = (Qc(1)-Qc0)/dx;
    E2 = norm(Kc*(-DeltaPc+RT*(Cc-Cs)) - dQcdx');
    E = E1 + E2;
  end
  Eprev = E;
%   The following few lines of code can be uncommened in order to observe
%   how the algorithm proceeds. With these commands uncommented, it is
%   necessary for the user to press the return key to move the
%   iteration forward every 100 steps.
%
%   [E count ep Ecurr Eprev]
%   if mod(count,100) == 0
%     figure(1); plot((1:N),Cs,(1:N),Cc,(1:N),Cd,(1:N),Ca); grid; 
%     figure(2); plot((1:N),Qs,(1:N),Qc,(1:N),Qd); grid; pause
%   end
%     
end


